% Generates results for Table A8 (model selection tests based on Feng, Giglio, and Xiu, JF, 2020, "Taming the Factor Zoo")
%
% Requires code and data from Feng, Giglio, and Xiu, which can be downloaded from https://doi.org/10.1111/jofi.12883
% (This script is based on their main.m file, with tuning parameter code provided separately by the authors)
% 
% If using a newer version of MATLAB, you may also need to download an updated version of the glmnet package, 
% available at https://hastie.su.domains/glmnet_matlab/download.html
%
% Tested on MATLAB R2022a

clear;
close all;

addpath(genpath('..'));

% fix random seed
seed_num = 100;
cv_N = 200;

% import factors
allfactors = csvread('zoofactors.csv',1,0); % their factor zoo returns, appended with our smb_dur and dur factor returns
date = allfactors(:,1);
rf = allfactors(:,2);
factors = allfactors(:,3:end);
L = length(date);
P = size(factors,2);

% test portfolios
port_3x2 = csvread('port_3x2.csv',0,1); % need to download from Feng, Giglio, and Xiu
port_3x2 = port_3x2 - rf*ones(1,size(port_3x2,2)); % excess return

% other information
summary = readtable('summary.csv');
factorname = summary.Row;
factorname_full = summary.Descpription;
year_pub = summary.Year;
year_end = summary.Year_end;

port_3x2_id = readtable('port_3x2_id.csv');

mkt_ind = find(ismember(factorname,'MktRf'));
smb_ind = find(ismember(factorname,'SMB'));
hml_ind = find(ismember(factorname,'HML'));

% form a smaller set of portfolios for bivariate sorted porfolios
kk = 10; % minimum number of stocks in a portfolio
include_3x2 = find(port_3x2_id.min_stk6>=kk)';
port_3x2b = [];
for i = 1:size(port_3x2_id,1)
    if ismember(i,include_3x2)
        port_3x2b = [port_3x2b port_3x2(:,(i*6-5):(i*6))];
    end
end
Ri = port_3x2b(:,1:750); % test assets

% load tune_center for 200 random seeds selected by cross-validations [recalculated below]
load tune_main.mat
tune_dur = zeros(1,2);

% choose control factors (all of their factors aside from divi and divo, which have multiple years of missing data, and adding our smb_dur)
ContrlList = [find(year_pub <= 2017);151]; % include 151 (our smb_dur)
ControlFactor = factors(:,ContrlList);
FF3 = factors(:,[mkt_ind,smb_ind,hml_ind])';

% test our factor (change if want more)
TestList = 152; % this is duration factor's row number
TestFactor = factors(:,TestList);

rst = [];
cvfull = 1; % if zero, don't rerun cross-validation (change to zero to run more quickly using previously calculated tuning parameters)

% test factor individually
for j = 1:length(TestList)

    disp(j);

    gt = TestFactor(:,j)'; % test factor
    ht = ControlFactor'; % control factor
    
    % use the average tuning parameter from 200 random seeds
    if cvfull==1
        tune_alt_200 = NaN(cv_N,2);
        for seednum = 1:cv_N
            if seednum==1 || mod(seednum,10)==0
                disp(seednum)
            end
            result = DS_TSCV(Ri', gt, ht, 1, 100, seednum, 10);
            if seednum==1 || mod(seednum,10)==0
                disp(result)
            end
            tune_alt_200(seednum,1) = result.tune1;
            tune_alt_200(seednum,2) = result.tune2;
        end
        tune_center(j,:) = [exp(-mean(-log(tune_alt_200(:,1)))) exp(-mean(-log(tune_alt_200(:,2))))];
        tune_dur = tune_center(j,:);
        save('tune_dur.mat','tune_dur');
    else
        load tune_dur.mat
        tune_center(j,:) = [tune_dur(j,1) tune_dur(j,2)];
    end
    
    model_ds = DS(Ri', gt, ht,-log(tune_center(j,1)),-log(tune_center(j,2)),1,seed_num); 
    tstat_ds = model_ds.lambdag_ds/model_ds.se_ds;
    pval_ds = 2*normcdf(-abs(tstat_ds),0,1);
    lambda_ds = model_ds.gamma_ds(1);
    factors_ds = size(model_ds.select,2);
    smb_incl_ds = ismember(151,model_ds.select);

    % single selection results, replace with a huge tune2
    model_ss = DS(Ri', gt, ht,-log(tune_center(j,1)),-log(1),1,seed_num);
    tstat_ss = model_ss.lambdag_ds/model_ss.se_ds;
    lambda_ss = model_ss.gamma_ds(1);

    % controlling everything, no selection, OLS
    model_ols = PriceRisk_OLS(Ri', gt, ht);
    tstat_ols = model_ols.lambdag_ols/model_ols.se_ols;
    lambda_ols = model_ols.lambda_ols(1);

    % only control for FF3 factors by OLS
    model_FF3 = PriceRisk_OLS(Ri', gt, FF3);
    tstat_FF3 = model_FF3.lambdag_ols/model_FF3.se_ols;
    lambda_FF3 = model_FF3.lambda_ols(1);

    % time series average
    avg = nanmean(gt);
    tstat_avg = avg/nanstd(gt)*sqrt(sum(~isnan(gt)));

    % combine the results in a table
    temp = table(tstat_ds, lambda_ds,pval_ds,factors_ds,smb_incl_ds,tstat_ss,lambda_ss,avg,tstat_avg,...
        lambda_ols,tstat_ols,lambda_FF3,tstat_FF3);

    rst = [rst;temp];
end
result = rst;


% extract outputs
% disp(factorname_full(model_ds.sel1))
lambda_ds = round(result.lambda_ds*10000,1); % bp
tstat_ds = round(result.tstat_ds,2);
pval_ds = round(result.pval_ds,4);
factors_ds = result.factors_ds;
smb_incl_ds = result.smb_incl_ds;

lambda_ss = round(result.lambda_ss*10000,1); % bp
tstat_ss = round(result.tstat_ss,2);

lambda_FF3 = round(result.lambda_FF3*10000,1); % bp
tstat_FF3 = round(result.tstat_FF3,2);

avg = round(result.avg*10000,1); % bp
tstat_avg = round(result.tstat_avg,2);

lambda_ols = round(result.lambda_ols*10000,1); % bp
tstat_ols = round(result.tstat_ols,2);

% factor names for our factors
factornames = factorname_full(TestList);

result = table(TestList,factornames,lambda_ds,tstat_ds,pval_ds,factors_ds,smb_incl_ds,lambda_ss,tstat_ss,lambda_FF3,...
    tstat_FF3,lambda_ols,tstat_ols,avg,tstat_avg);

% display the table
disp(result)

% output table as a CSV file
writetable(result, 'zooresults.csv');
